/*
 * ri.h
 *
 * Resolved Instruments DPD80 interface library
 *
 * Copyright 2016 Resolved Instruments Inc.
*/

#ifndef RI_H
#define RI_H
#ifdef __cplusplus
extern "C" {
#endif


#if defined _WIN32
	#if defined LIBRI_STATIC
		#define LIBRI_API
	#elif defined LIBRI_EXPORT
		#define LIBRI_API __declspec(dllexport)
	#else
		#define LIBRI_API __declspec(dllimport)
	#endif
#else
	#define LIBRI_API
#endif


#include "stdint.h"
typedef unsigned int uint;
#include "ri_ports.h"


typedef enum {
	RI_SPEED_UNKNOWN,
	RI_SPEED_LOW,
	RI_SPEED_FULL,
	RI_SPEED_HIGH,
	RI_SPEED_SUPER,
} RI_USB_SPEED_t;


typedef enum {
	RI_SUCCESS = 0,
	RI_ERROR = -1,
	RI_ERROR_USB = -2,
	RI_ERROR_PARAMS = -3,
} RI_ERROR_t;



/*
 * The device in use is stored as an opaque pointer with the type ri_device*.
 * When ri_device* is 0, no device exists and it's considered unopened.
 *
 * When a ri_device* is not equal to zero, it is assumed to be an opened device.
 * Therefore, always initialize the device pointer to 0 on creation.
*/
typedef struct ri_device ri_device;


/*
 * ri_init and ri_exit initialize and deinitialize the libri library.
 * ri_init() should be called before any additional libri functions
 * are called.
 *
 * ri_exit should be called at the end of the application.
 *
 */
LIBRI_API int ri_init(void);
LIBRI_API void ri_exit(void);


/*
 * ri_open_device finds the first available ri device and opens it for further
 * commands.
 *
 * If ri_open_device returns 0, no devices were able to be found.  If it returns
 * a non-zero value a device has been opened which should eventually be closed with
 * ri_close_device.
 *
 * ri_close_device closes an open ri_device*.  It always returns 0 as a convenient
 * way to zero the ri_device* after closeing.
 */
LIBRI_API ri_device* ri_open_device(void);
//LIBRI_API ri_device* ri_find_device(const char *serial);
LIBRI_API ri_device* ri_close_device(ri_device *dev);


/*
 * ri_get_raw_data is the simplest method to get samples from the DPD80.
 *
 * nsamples is the number of samples to get. *buff is a pointer to where those
 * samples should be stored.
*/
LIBRI_API int ri_get_raw_data(ri_device *dev, uint64_t nsamples, uint16_t *buff);

/*
 * ri_start_continuous_transfer allows data to be transferred for potentially unlimited
 * period of time. A callback function with the same type as ri_transfer_callback should
 * be supplied which is called everytime new data is available.
 *
 * data transfer will continue until the callback function returns false.
 *
 * see examples/2_stream/ for an example on how to use this function.
 */
typedef int(*ri_transfer_callback)(uint16_t* data, int ndata, int dataloss, void *userdata);
LIBRI_API int ri_start_continuous_transfer(ri_device *dev, ri_transfer_callback callback, void *userdata);



/*
 * Returns the USB speed of the open ri_device*.  Data transfer operations
 * are only supported when the device is RI_SPEED_SUPER i.e. when in USB 3 mode.
 *
 */
LIBRI_API RI_USB_SPEED_t ri_get_usb_speed(ri_device *dev);

/*
 * The ri_set_ functions enable or disable various features / modes of the DPD80.
 * enable is a boolean argument (1 or 0) that enables the specific feature.
 *
 * these features are rest to their default value when the DPD80 is reset.
 *
 * highgain: turns on the highgain mode of the DPD80.  default 1.
 * antialias: turns on the 35 MHz antialias filter on the DPD80. default 1.
 * powerdown: powers down all the non-usb features on the DPD80.  default 0.
 * adcclock: turns on the 80 MHz ADC clock.  Usefull to turn off if using the analog
 *   output of the DPD80 and want to get rid of the 80 MHz clock peak. default 1.
 */
LIBRI_API int ri_set_highgain(ri_device *dev, int enable);
LIBRI_API int ri_set_antialias(ri_device *dev, int enable);
LIBRI_API int ri_set_powerdown(ri_device *dev, int enable);
LIBRI_API int ri_set_adcclock(ri_device *dev, int enable);

/*
 * The ri_read_ functions read the current state of the features and return
 * wether they are on (1) or off (0).
 */
LIBRI_API int ri_read_highgain(ri_device *dev);
LIBRI_API int ri_read_antialias(ri_device *dev);
LIBRI_API int ri_read_powerdown(ri_device *dev);
LIBRI_API int ri_read_adcclock(ri_device *dev);

/*
 * ri_port_ functions control the A, B, S, and T ports on the back of the DPD80.
 *
 * ports are addresable using the values
 *   RI_PORT_A
 *	 RI_PORT_B
 *	 RI_PORT_S
 *	 RI_PORT_T
 *
 * currently the S and T ports can only be configured in STREAM_IN mode where the
 * 3.3V TTL signals into these ports are recorded with the ADC samples in bits
 * 16 and 15.  This should be upgradable with future software updates to configure
 * the S and T pins to act as general purpose ports too.
 *
 * ri_port_drive outputs the 3.3V TTL signal specified by value (1 or 0) on the port.
 * ri_port_read reads the 3.3V TTL signal applied to the port.
 * ri_port_pwm causes the port to output a square wave signal with the period
 *   of threshold clock cycles (clocked at ~400 MHz), and a duty cycle of threshold
 *   clock cycles.
 * ri_port_pulse causes a single pulse to be output on the port with a length of
 *   threshold clock cycles (clocked at ~400 MHz).
 *
 */
LIBRI_API int ri_port_drive(ri_device *dev, RI_PORT_t port, int value);
LIBRI_API int ri_port_read(ri_device *dev, RI_PORT_t port);
LIBRI_API int ri_port_pwm(ri_device *dev, RI_PORT_t port, uint32_t threshold, uint32_t period);
LIBRI_API int ri_port_pulse(ri_device *dev, RI_PORT_t port, uint32_t threshold);


/*
* the ri_test_ functions set the adc to output predefined test values to verify
* proper behavior.  Shouldn't need to be used by end users.
*
* ri_test_normal returns the device to normal operation.
*/
LIBRI_API void ri_test_pn9(ri_device *dev);
LIBRI_API void ri_test_checkerboard(ri_device *dev);
LIBRI_API void ri_test_user(ri_device *dev, uint16_t sample);
LIBRI_API void ri_test_normal(ri_device *dev);



#ifdef __cplusplus
}
#endif
#endif // RI_H
